@@ -1,5 +1,5 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2000,2001,2002,2003 Free Software Foundation, Inc.
+   Copyright (C) 2000,2001,2002,2003,2004 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
   
@@ -24,8 +24,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-#include <radius.h>
-#include <mem.h>
+#include <common.h>
 
 /* From rfc 2138:
       Call the shared secret S and the pseudo-random 128-bit Request
@@ -71,13 +70,13 @@
 /* General purpose encryption and decryption functions. These satisfy
    both rfc 2138 and 2868. */
 void
-encrypt_text(u_char **encr_text,
-	     size_t *encr_size,
-	     u_char *password,   /* Cleantext password */
-	     u_char *vector,     /* Request authenticator */
-	     u_char *secret,     /* Shared secret */
-	     u_char *salt,
-	     size_t saltlen)
+grad_encrypt_text(u_char **encr_text,
+		  size_t *encr_size,
+		  u_char *password,   /* Cleantext password */
+		  u_char *authenticator,     /* Request authenticator */
+		  u_char *secret,     /* Shared secret */
+		  u_char *salt,
+		  size_t saltlen)
 {
         int passlen;
         int secretlen;
@@ -86,15 +85,15 @@ encrypt_text(u_char **encr_text,
         u_char *passbuf;
         int md5len;
         u_char *md5buf;
-        u_char digest[AUTH_VECTOR_LEN];
+        u_char digest[GRAD_AUTHENTICATOR_LENGTH];
         u_char *cp;
         int i, j;
         
         passlen = strlen(password);
-        nchunks = (passlen + AUTH_VECTOR_LEN - 1) / AUTH_VECTOR_LEN;
-        buflen = nchunks * AUTH_VECTOR_LEN;
+        nchunks = (passlen + GRAD_AUTHENTICATOR_LENGTH - 1) / GRAD_AUTHENTICATOR_LENGTH;
+        buflen = nchunks * GRAD_AUTHENTICATOR_LENGTH;
 
-        *encr_text = emalloc(buflen);
+        *encr_text = grad_emalloc(buflen);
         *encr_size = buflen;
         passbuf = *encr_text;
 
@@ -103,41 +102,42 @@ encrypt_text(u_char **encr_text,
         memcpy(passbuf, password, passlen);
 
         secretlen = strlen(secret);
-        md5len = secretlen + AUTH_VECTOR_LEN;
-        md5buf = emalloc(md5len + saltlen);
+        md5len = secretlen + GRAD_AUTHENTICATOR_LENGTH;
+        md5buf = grad_emalloc(md5len + saltlen);
         memcpy(md5buf, secret, secretlen);
 
-        cp = vector;
+        cp = authenticator;
         for (i = 0; i < buflen; ) {
                 /* Compute next MD5 hash */
-                memcpy(md5buf + secretlen, cp, AUTH_VECTOR_LEN);
+                memcpy(md5buf + secretlen, cp, GRAD_AUTHENTICATOR_LENGTH);
 		if (i == 0 && saltlen) {
 			memcpy(md5buf + md5len, salt, saltlen);
-			md5_calc(digest, md5buf, md5len + saltlen);
+			grad_md5_calc(digest, md5buf, md5len + saltlen);
 		} else
-			md5_calc(digest, md5buf, md5len);
+			grad_md5_calc(digest, md5buf, md5len);
                 /* Save hash start */
                 cp = passbuf + i;
                 /* Encrypt next chunk */
-                for (j = 0; j < AUTH_VECTOR_LEN; j++, i++)
+                for (j = 0; j < GRAD_AUTHENTICATOR_LENGTH; j++, i++)
                         passbuf[i] ^= digest[j];
         }
-        efree(md5buf);
+        grad_free(md5buf);
 }
 
 void
-decrypt_text(u_char *password,   /* At least AUTH_STRING_LEN+1
-				    characters long */
-	     u_char *encr_text,  /* encrypted text */
-	     size_t encr_size,   /* Size of encr_text and password buffers */
-	     u_char *vector,     /* Request authenticator */
-	     u_char *secret,     /* Shared secret */
-	     u_char *salt,
-	     size_t saltsize)
+grad_decrypt_text(u_char *password,   /* At least GRAD_STRING_LENGTH+1
+					 characters long */
+		  u_char *encr_text,  /* encrypted text */
+		  size_t encr_size,   /* Size of encr_text and password
+					 buffers */
+		  u_char *authenticator,     /* Request authenticator */
+		  u_char *secret,     /* Shared secret */
+		  u_char *salt,
+		  size_t saltsize)
 {
         int md5len;
         u_char *md5buf;
-        u_char digest[AUTH_VECTOR_LEN];
+        u_char digest[GRAD_AUTHENTICATOR_LENGTH];
         u_char *cp;
         int secretlen;
         int i, j;
@@ -147,60 +147,60 @@ decrypt_text(u_char *password,   /* At least AUTH_STRING_LEN+1
         
         /* Prepare md5buf */
         secretlen = strlen(secret);
-        md5len = secretlen + AUTH_VECTOR_LEN;
-        md5buf = emalloc(md5len);
+        md5len = secretlen + GRAD_AUTHENTICATOR_LENGTH;
+        md5buf = grad_emalloc(md5len);
         memcpy(md5buf, secret, secretlen);
 
-        cp = vector;
+        cp = authenticator;
         for (i = 0; i < encr_size; ) {
                 /* Compute next MD5 hash */
-                memcpy(md5buf + secretlen, cp, AUTH_VECTOR_LEN);
+                memcpy(md5buf + secretlen, cp, GRAD_AUTHENTICATOR_LENGTH);
 		if (i == 0 && saltsize) {
 			memcpy(md5buf + md5len, salt, saltsize);
-			md5_calc(digest, md5buf, md5len + saltsize);
+			grad_md5_calc(digest, md5buf, md5len + saltsize);
 		} else
-			md5_calc(digest, md5buf, md5len);
+			grad_md5_calc(digest, md5buf, md5len);
                 /* Save hash start */
                 cp = encr_text + i;
                 /* Decrypt next chunk */
-                for (j = 0; j < AUTH_VECTOR_LEN; j++, i++)
+                for (j = 0; j < GRAD_AUTHENTICATOR_LENGTH; j++, i++)
                         password[i] ^= digest[j];
         }
         password[encr_size] = 0;
-        efree(md5buf);
+        grad_free(md5buf);
 }
 
 
 /* RFC 2138 functions */
 void
-encrypt_password(VALUE_PAIR *pair,
-		 char *password, /* Cleantext password */
-		 char *vector,   /* Request authenticator */
-		 char *secret)   /* Shared secret */
+grad_encrypt_password(grad_avp_t *pair,
+		      char *password, /* Cleantext password */
+		      char *authenticator,   /* Request authenticator */
+		      char *secret)   /* Shared secret */
 {
 	u_char *encr_text;
 	size_t encr_size;
 	
-	encrypt_text(&encr_text, &encr_size,
-		     password, vector, secret, 
-		     NULL, 0);
+	grad_encrypt_text(&encr_text, &encr_size,
+			  password, authenticator, secret, 
+			  NULL, 0);
         pair->avp_strvalue = encr_text;
         pair->avp_strlength = encr_size;
 }
 
 void
-decrypt_password(char *password,   /* At least AUTH_STRING_LEN+1
-				      characters long */
-		 VALUE_PAIR *pair, /* Password pair */
-		 char *vector,     /* Request authenticator */
-		 char *secret)     /* Shared secret */
+grad_decrypt_password(char *password,   /* At least GRAD_STRING_LENGTH+1
+					   characters long */
+		      grad_avp_t *pair, /* Password pair */
+		      char *authenticator,     /* Request authenticator */
+		      char *secret)     /* Shared secret */
 {
-	decrypt_text(password,
-		     pair->avp_strvalue,
-		     pair->avp_strlength,
-		     vector,
-		     secret, 
-		     NULL,
+	grad_decrypt_text(password,
+			  pair->avp_strvalue,
+			  pair->avp_strlength,
+			  authenticator,
+			  secret, 
+			  NULL,
 		     0);
 }
 
@@ -208,15 +208,15 @@ decrypt_password(char *password,   /* At least AUTH_STRING_LEN+1
    Decrypt a password encrypted using broken algorythm.
    This is for use with such brain-damaged NASes as MAX ascend. */
 void
-decrypt_password_broken(char *password, /* At least AUTH_STRING_LEN+1
-					   characters long */
-			VALUE_PAIR *pair, /* Password pair */
-			char *vector,     /* Request authenticator */
-			char *secret)     /* Shared secret */
+grad_decrypt_password_broken(char *password, /* At least GRAD_STRING_LENGTH+1
+						characters long */
+			     grad_avp_t *pair, /* Password pair */
+			     char *authenticator,     /* Request authenticator */
+			     char *secret)     /* Shared secret */
 {
         int md5len;
         char *md5buf;
-        char digest[AUTH_VECTOR_LEN];
+        char digest[GRAD_AUTHENTICATOR_LENGTH];
         int secretlen;
         int passlen;
         int i, j;
@@ -228,71 +228,71 @@ decrypt_password_broken(char *password, /* At least AUTH_STRING_LEN+1
         
         /* Prepare md5buf */
         secretlen = strlen(secret);
-        md5len = secretlen + AUTH_VECTOR_LEN;
-        md5buf = emalloc(md5len);
+        md5len = secretlen + GRAD_AUTHENTICATOR_LENGTH;
+        md5buf = grad_emalloc(md5len);
         memcpy(md5buf, secret, secretlen);
 
         /* Compute next MD5 hash */
-        memcpy(md5buf + secretlen, vector, AUTH_VECTOR_LEN);
-        md5_calc(digest, md5buf, md5len);
+        memcpy(md5buf + secretlen, authenticator, GRAD_AUTHENTICATOR_LENGTH);
+        grad_md5_calc(digest, md5buf, md5len);
 
         for (i = 0; i < passlen; ) {
                 /* Decrypt next chunk */
-                for (j = 0; j < AUTH_VECTOR_LEN; j++, i++)
+                for (j = 0; j < GRAD_AUTHENTICATOR_LENGTH; j++, i++)
                         password[i] ^= digest[j];
         }
-        efree(md5buf);
+        grad_free(md5buf);
 }
 	
 
 /* RFC 2868 */
       
 void
-encrypt_tunnel_password(VALUE_PAIR *pair,
-			u_char tag,
-			char *password, /* Cleantext password */
-			char *vector,   /* Request authenticator */
-			char *secret)   /* Shared secret */
+grad_encrypt_tunnel_password(grad_avp_t *pair,
+			     u_char tag,
+			     char *password, /* Cleantext password */
+			     char *authenticator,   /* Request authenticator */
+			     char *secret)   /* Shared secret */
 {
 	u_char *encr_text;
 	size_t encr_size;
 	char *plaintext;
 	size_t length = strlen(password);
-	unsigned short salt = htons( (((long)pair ^ *(long *)vector) & 0xffff)
+	unsigned short salt = htons( (((long)pair ^ *(long *)authenticator) & 0xffff)
 				     | 0x8000 );
 	
-	plaintext = emalloc(length+2);
+	plaintext = grad_emalloc(length+2);
 	plaintext[0] = length;
 	memcpy(&plaintext[1], password, length + 1);
-	encrypt_text(&encr_text, &encr_size,
-		     plaintext, vector, secret, 
-		     (u_char*) &salt, 2);
-	efree(plaintext);
+	grad_encrypt_text(&encr_text, &encr_size,
+			  plaintext, authenticator, secret, 
+			  (u_char*) &salt, 2);
+	grad_free(plaintext);
 	
 	pair->avp_strlength = 3 + encr_size;
-	pair->avp_strvalue = emalloc(pair->avp_strlength);
+	pair->avp_strvalue = grad_emalloc(pair->avp_strlength);
 	pair->avp_strvalue[0] = tag;
 	memcpy(&pair->avp_strvalue[1], &salt, 2);
 	memcpy(&pair->avp_strvalue[3], encr_text, encr_size);
-	efree(encr_text);
+	grad_free(encr_text);
 }
 
 void
-decrypt_tunnel_password(char *password,   /* At least AUTH_STRING_LEN+1
-					     characters long */
-			u_char *tag,
-			VALUE_PAIR *pair, /* Password pair */
-			char *vector,     /* Request authenticator */
-			char *secret)     /* Shared secret */
+grad_decrypt_tunnel_password(char *password,   /* At least GRAD_STRING_LENGTH+1
+						  characters long */
+			     u_char *tag,
+			     grad_avp_t *pair, /* Password pair */
+			     char *authenticator,     /* Request authenticator */
+			     char *secret)     /* Shared secret */
 {
 	size_t length;
-	decrypt_text(password,
-		     pair->avp_strvalue + 3,
-		     pair->avp_strlength - 3,
-		     vector,
-		     secret, 
-		     &pair->avp_strvalue[1],
-		     2);
+	grad_decrypt_text(password,
+			  pair->avp_strvalue + 3,
+			  pair->avp_strlength - 3,
+			  authenticator,
+			  secret, 
+			  &pair->avp_strvalue[1],
+			  2);
 	length = *(u_char*) password;
 	memmove(password, password + 1, length);
 	password[length] = 0;
