@@ -1,5 +1,5 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2000,2001,2002,2003 Free Software Foundation, Inc.
+   Copyright (C) 2000,2001,2002,2003,2004 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
   
@@ -25,15 +25,15 @@
 #include <stdio.h>
 #include <errno.h>
 #include <ctype.h>
-#include <argcv.h>
-#include <radius.h>
+#include <common.h>
 
 #if !HAVE_DECL_GETLINE
 int getline (char **lineptr, size_t *n, FILE *stream);
 #endif
 
 int
-read_raddb_file(char *filename, int vital, int (*fun)(), void *closure)
+grad_read_raddb_file(char *filename, int vital, char *delim,
+		     int (*fun)(), void *closure)
 {
         int    argc;
         char **argv;
@@ -41,21 +41,23 @@ read_raddb_file(char *filename, int vital, int (*fun)(), void *closure)
         char *lineptr = NULL;
         size_t bsize = 0;
         int nread;
-        LOCUS loc;
+        grad_locus_t loc;
 	
         input = fopen(filename, "r");
         if (!input) {
                 if (vital) {
-                        radlog(L_ERR|L_PERROR, _("can't open file `%s'"),
-                               filename);
+                        grad_log(L_ERR|L_PERROR, _("can't open file `%s'"),
+                                 filename);
                         return -1;
                 } else {
-                        radlog(L_NOTICE|L_PERROR, _("can't open file `%s'"),
-                               filename);
+                        grad_log(L_NOTICE|L_PERROR, _("can't open file `%s'"),
+                                 filename);
                         return 0;
                 }
         }
 
+	if (!delim)
+		delim = "";
 	loc.file = filename;
         loc.line = 0;
         while (getline(&lineptr, &bsize, input) > 0) {
@@ -68,7 +70,7 @@ read_raddb_file(char *filename, int vital, int (*fun)(), void *closure)
 		}
 		if (lineptr[0] == 0)
 			continue;
-                if (argcv_get(lineptr, "", NULL, &argc, &argv) == 0) {
+                if (argcv_get(lineptr, delim, NULL, &argc, &argv) == 0) {
                         int n;
                         for (n = 0; n < argc && argv[n][0] != '#'; n++)
                                 ;
