@@ -1,5 +1,7 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2000,2001,2003 Sergey Poznyakoff
+   Copyright (C) 2001,2002,2003 Free Software Foundation, Inc.
+
+   Written by Sergey Poznyakoff
  
    This file is free software; as a special exception the author gives
    unlimited permission to copy and/or distribute it, with or without
@@ -10,9 +12,7 @@
    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. */
 
 int
-strcspn(s1, s2)
-        char *s1;
-        char *s2;
+strcspn(char *s1, char *s2)
 {
         register char *scan1;
         register char *scan2;
