@@ -1,5 +1,5 @@
 
-/*  A Bison parser, made from /home/gray/radius/lib/./users.y
+/*  A Bison parser, made from /home/gray/gnu/radius/lib/./users.y
     by GNU Bison version 1.28  */
 
 #define YYBISON 1  /* Identify Bison output.  */
@@ -18,7 +18,7 @@
 #line 1 "users.y"
 
 /* This file is part of GNU Radius.
-   Copyright (C) 2000,2001,2002,2003 Free Software Foundation, Inc.
+   Copyright (C) 2000,2001,2002,2003,2004 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
   
@@ -56,31 +56,29 @@
 #include <errno.h>
 #include <sys/wait.h>
 
-#include <sysdep.h>
-#include <radius.h>
-#include <parser.h>
+#include <common.h>
 
 #define YYMAXDEPTH 10
 
-static LOCUS start_loc;
+static grad_locus_t start_loc;
 
 static void *closure;
 static register_rule_fp add_entry;
-static VALUE_PAIR *install_pair0(char *name, int op, char *valstr);
+static grad_avp_t *grad_create_pair0(char *name, int op, char *valstr);
 
 int uyyerror(char *s);
 extern int uyylex();
  
 
-#line 69 "users.y"
+#line 67 "users.y"
 typedef union {
         char *string;
-        MATCHING_RULE *rule;
+        grad_matching_rule_t *rule;
         struct {
-                VALUE_PAIR *lhs, *rhs;
+                grad_avp_t *lhs, *rhs;
         } descr;
-        VALUE_PAIR *pair;
-        int op;
+        grad_avp_t *pair;
+        enum grad_operator op;
 } YYSTYPE;
 #ifndef YYDEBUG
 #define YYDEBUG 1
@@ -152,9 +150,9 @@ static const short uyyrhs[] = {    -1,
 
 #if YYDEBUG != 0
 static const short uyyrline[] = { 0,
-    81,    82,    85,    88,    89,    95,    99,   111,   117,   124,
-   128,   131,   132,   143,   147,   153,   157,   161,   165,   169,
-   173,   179,   180
+    79,    80,    83,    86,    87,    93,    97,   109,   115,   122,
+   126,   129,   130,   141,   145,   151,   155,   159,   163,   167,
+   171,   177,   178
 };
 #endif
 
@@ -758,27 +756,27 @@ uyyreduce:
   switch (uyyn) {
 
 case 3:
-#line 86 "users.y"
+#line 84 "users.y"
 {
            ;
     break;}
 case 5:
-#line 90 "users.y"
+#line 88 "users.y"
 {
-                   users_sync(); uyyerrok; uyyclearin;
+                   grad_parser_lex_sync(); uyyerrok; uyyclearin;
            ;
     break;}
 case 6:
-#line 96 "users.y"
+#line 94 "users.y"
 {
                    add_entry(closure, &start_loc, uyyvsp[-1].string, uyyvsp[0].descr.lhs, uyyvsp[0].descr.rhs);
            ;
     break;}
 case 7:
-#line 100 "users.y"
+#line 98 "users.y"
 {
-                   radlog(L_ERR, _("discarding user `%s'"), uyyvsp[-1].string);
-                   if (users_sync() <= 0)
+                   grad_log(L_ERR, _("discarding user `%s'"), uyyvsp[-1].string);
+                   if (grad_parser_lex_sync() <= 0)
 			   uyychar = 0; /* force end-of-file */
 		   else {
 			   uyyerrok;
@@ -787,81 +785,81 @@ case 7:
            ;
     break;}
 case 8:
-#line 112 "users.y"
+#line 110 "users.y"
 {
-		   start_loc = source_locus;
+		   start_loc = grad_parser_source_locus;
            ;
     break;}
 case 9:
-#line 118 "users.y"
+#line 116 "users.y"
 {
                    uyyval.descr.lhs = uyyvsp[-1].pair;
                    uyyval.descr.rhs = uyyvsp[0].pair;
            ;
     break;}
 case 10:
-#line 125 "users.y"
+#line 123 "users.y"
 {
                    uyyval.pair = NULL;
            ;
     break;}
 case 13:
-#line 133 "users.y"
+#line 131 "users.y"
 {
                    if (uyyvsp[-2].pair) {
                            if (uyyvsp[0].pair) 
-                                   avl_add_list(&uyyvsp[-2].pair, uyyvsp[0].pair);
+                                   grad_avl_add_list(&uyyvsp[-2].pair, uyyvsp[0].pair);
                            uyyval.pair = uyyvsp[-2].pair;
                    } else
                            uyyval.pair = uyyvsp[0].pair;
            ;
     break;}
 case 14:
-#line 144 "users.y"
+#line 142 "users.y"
 {
-                   uyyval.pair = install_pair0(uyyvsp[-2].string, uyyvsp[-1].op, uyyvsp[0].string);   
+                   uyyval.pair = grad_create_pair0(uyyvsp[-2].string, uyyvsp[-1].op, uyyvsp[0].string);   
            ;
     break;}
 case 15:
-#line 148 "users.y"
+#line 146 "users.y"
 {
                    YYERROR;
            ;
     break;}
 case 16:
-#line 154 "users.y"
+#line 152 "users.y"
 {
-                   uyyval.op = OPERATOR_EQUAL;
+                   uyyval.op = grad_operator_equal;
            ;
     break;}
 case 17:
-#line 158 "users.y"
+#line 156 "users.y"
 {
-                   uyyval.op = OPERATOR_LESS_THAN;
+                   uyyval.op = grad_operator_less_than;
            ;
     break;}
 case 18:
-#line 162 "users.y"
+#line 160 "users.y"
 { 
-                   uyyval.op = OPERATOR_GREATER_THAN;
+                   uyyval.op = grad_operator_greater_than;
            ;
     break;}
 case 19:
-#line 166 "users.y"
+#line 164 "users.y"
 {
-                   uyyval.op = OPERATOR_NOT_EQUAL;
+                   uyyval.op = grad_operator_not_equal;
            ;
     break;}
 case 20:
-#line 170 "users.y"
+#line 168 "users.y"
 {
-                   uyyval.op = OPERATOR_LESS_EQUAL;
+                   uyyval.op = grad_operator_less_equal;
            ;
     break;}
 case 21:
-#line 174 "users.y"
+#line 172 "users.y"
 {
-                   uyyval.op = OPERATOR_GREATER_EQUAL;
+                   uyyval.op = grad_operator_greater_equal;
            ;
     break;}
 }
@@ -1086,40 +1084,41 @@ uyyerrhandle:
     }
   return 1;
 }
-#line 183 "users.y"
+#line 181 "users.y"
 
 
 int
 uyyerror(char *s)
 {
-	radlog_loc(L_ERR, &source_locus, "%s", s);
+	grad_log_loc(L_ERR, &grad_parser_source_locus, "%s", s);
 	return 0;
 }
 
-VALUE_PAIR *
-install_pair0(char *name, int op, char *valstr)
+grad_avp_t *
+grad_create_pair0(char *name, int op, char *valstr)
 {
-	return install_pair(&source_locus, name, op, valstr);
+	return grad_create_pair(&grad_parser_source_locus, name, op, valstr);
 }
 
-VALUE_PAIR *
-install_pair(LOCUS *loc, char *name, int op, char *valstr)
+grad_avp_t *
+grad_create_pair(grad_locus_t *loc, char *name, 
+                 enum grad_operator op, char *valstr)
 {
-        DICT_ATTR       *attr = NULL;
-        DICT_VALUE      *dval;
-        VALUE_PAIR      *pair, *pair2;
+        grad_dict_attr_t *attr = NULL;
+        grad_dict_value_t *dval;
+        grad_avp_t *pair, *pair2;
         char *s;
         int x;
         time_t timeval;
         struct tm *tm, tms;
         
-        if ((attr = attr_name_to_dict(name)) == (DICT_ATTR *)NULL) {
-                radlog_loc(L_ERR, loc, _("unknown attribute `%s'"),
-			   name);
+        if ((attr = grad_attr_name_to_dict(name)) == NULL) {
+                grad_log_loc(L_ERR, loc, _("unknown attribute `%s'"),
+			     name);
                 return NULL;
         }
 
-        pair = avp_alloc();
+        pair = grad_avp_alloc();
         
         pair->next = NULL;
         pair->name = attr->name;
@@ -1129,43 +1128,43 @@ install_pair(LOCUS *loc, char *name, int op, char *valstr)
         pair->operator = op;
 
         if (valstr[0] == '=') {
-                pair->eval_type = eval_interpret;
-                pair->avp_strvalue = estrdup(valstr+1);
+                pair->eval_type = grad_eval_interpret;
+                pair->avp_strvalue = grad_estrdup(valstr+1);
                 pair->avp_strlength = strlen(pair->avp_strvalue);
                 return pair;
         }
 
-        pair->eval_type = eval_const;
+        pair->eval_type = grad_eval_const;
         
         switch (pair->type) {
-        case TYPE_STRING:
+        case GRAD_TYPE_STRING:
                 if (pair->attribute == DA_EXEC_PROGRAM ||
                     pair->attribute == DA_EXEC_PROGRAM_WAIT) {
                         if (valstr[0] != '/' && valstr[0] != '|') {
-                                radlog_loc(L_ERR, loc,
-					   _("%s: not an absolute pathname"),
-					   name);
-                                avp_free(pair);
+                                grad_log_loc(L_ERR, loc,
+					     _("%s: not an absolute pathname"),
+					     name);
+                                grad_avp_free(pair);
                                 return NULL;
                         }
                 }
 
-		pair->avp_strvalue = estrdup(valstr);
+		pair->avp_strvalue = grad_estrdup(valstr);
                 pair->avp_strlength = strlen(pair->avp_strvalue);
 		
 		if (attr->parser && attr->parser(pair, &s)) {
-			radlog_loc(L_ERR, loc,
-				   "%s %s: %s",
-				   _("attribute"),
-				   pair->name, s);
+			grad_log_loc(L_ERR, loc,
+				     "%s %s: %s",
+				     _("attribute"),
+				     pair->name, s);
 			free(s);
-			avp_free(pair);
+			grad_avp_free(pair);
 			return NULL;
 		}
 
                 break;
 
-        case TYPE_INTEGER:
+        case GRAD_TYPE_INTEGER:
                 /*
                  *      For DA_NAS_PORT_ID, allow a
                  *      port range instead of just a port.
@@ -1175,28 +1174,30 @@ install_pair(LOCUS *loc, char *name, int op, char *valstr)
                                 if (!isdigit(*s))
                                         break;
                         if (*s) {
-                                pair->type = TYPE_STRING;
-                                pair->avp_strvalue = estrdup(valstr);
+                                pair->type = GRAD_TYPE_STRING;
+                                pair->avp_strvalue = grad_estrdup(valstr);
                                 pair->avp_strlength = strlen(pair->avp_strvalue);
                                 break;
                         }
                 }
                 if (isdigit(*valstr)) {
                         pair->avp_lvalue = atoi(valstr);
-                } else if ((dval = value_name_to_value(valstr, pair->attribute)) == NULL) {
-                        avp_free(pair);
-                        radlog_loc(L_ERR, loc,
+                } else if ((dval = grad_value_name_to_value(valstr,
+							    pair->attribute))
+			         == NULL) {
+                        grad_avp_free(pair);
+                        grad_log_loc(L_ERR, loc,
 				_("value %s is not declared for attribute %s"),
-				   valstr, name);
+				     valstr, name);
                         return NULL;
                 } else {
                         pair->avp_lvalue = dval->value;
                 }
                 break;
 
-        case TYPE_IPADDR:
+        case GRAD_TYPE_IPADDR:
                 if (pair->attribute != DA_FRAMED_IP_ADDRESS) {
-                        pair->avp_lvalue = ip_gethostaddr(valstr);
+                        pair->avp_lvalue = grad_ip_gethostaddr(valstr);
                 } else {
                         /*
                          *      We allow a "+" at the end to
@@ -1212,41 +1213,41 @@ install_pair(LOCUS *loc, char *name, int op, char *valstr)
                                         x = 1;
                                 }
                         }
-                        pair->avp_lvalue = ip_gethostaddr(valstr);
+                        pair->avp_lvalue = grad_ip_gethostaddr(valstr);
 
 			if (x) {
 				char *s;
 				asprintf(&s, "%lu+%%{NAS-Port-Id}",
 					 pair->avp_lvalue);
-				pair->avp_strvalue = estrdup(s);
+				pair->avp_strvalue = grad_estrdup(s);
 				pair->avp_strlength = strlen(pair->avp_strvalue);
-				pair->eval_type = eval_interpret;
+				pair->eval_type = grad_eval_interpret;
 				free(s);
 			}
                 }
                 break;
                 
-        case TYPE_DATE:
+        case GRAD_TYPE_DATE:
                 timeval = time(0);
                 tm = localtime_r(&timeval, &tms);
-                if (user_gettime(valstr, tm)) {
-                        radlog_loc(L_ERR, loc, _("%s: can't parse date"),
-				   name);
-                        avp_free(pair);
+                if (grad_parse_time_string(valstr, tm)) {
+                        grad_log_loc(L_ERR, loc, _("%s: can't parse date"),
+				     name);
+                        grad_avp_free(pair);
                         return NULL;
                 }
 #ifdef TIMELOCAL
-                pair->avp_lvalue = (UINT4)timelocal(tm);
+                pair->avp_lvalue = (grad_uint32_t)timelocal(tm);
 #else /* TIMELOCAL */
-                pair->avp_lvalue = (UINT4)mktime(tm);
+                pair->avp_lvalue = (grad_uint32_t)mktime(tm);
 #endif /* TIMELOCAL */
                 break;
 
         default:
-                radlog_loc(L_ERR, loc,
-			   _("%s: unknown attribute type %d"),
-			   name, pair->type);
-                avp_free(pair);
+                grad_log_loc(L_ERR, loc,
+			     _("%s: unknown attribute type %d"),
+			     name, pair->type);
+                grad_avp_free(pair);
                 return NULL;
         }
 
@@ -1256,26 +1257,26 @@ install_pair(LOCUS *loc, char *name, int op, char *valstr)
 extern int uyydebug;
 
 int
-parse_file(char *file, void *c, register_rule_fp f)
+grad_parse_rule_file(char *file, void *c, register_rule_fp f)
 {
         int rc;
         
-        if (init_lex(file))
+        if (grad_parser_lex_init(file))
                 return -1;
         closure = c;
         add_entry = f;
 
         uyydebug = 0;
         rc = uyyparse();
-        done_lex();
+        grad_parser_lex_finish();
         return rc;
 }
 
 void
-enable_usr_dbg(int val)
+grad_enable_rule_debug(int val)
 {
         uyydebug = val;
-	radlog_loc(L_NOTICE, &source_locus,
-		   uyydebug ? _("enabled userfile parser debugging") :
-             		     _("disabled userfile parser debugging"));
+	grad_log_loc(L_NOTICE, &grad_parser_source_locus,
+		     uyydebug ? _("enabled userfile parser debugging") :
+             	 	       _("disabled userfile parser debugging"));
 }
