@@ -1,4 +1,3 @@
-#define EMBEDDED_EXPIRATION_INFO
 /* This file is part of GNU Radius
    Copyright (C) 2000,2001,2002,2003,2004 Free Software Foundation, Inc.
 
@@ -264,7 +263,8 @@ unix_pass(char *name, char *passwd)
  }
 #endif
         rc = grad_md5crypt(passwd, encrypted_pass, encpw, pwlen+1) == NULL
-                || memcmp(encpw, encrypted_pass, pwlen);
+                || strlen (encpw) != pwlen
+		|| memcmp(encpw, encrypted_pass, pwlen);
         grad_free(encpw);
 	grad_free(encrypted_pass);
         if (rc)
@@ -927,25 +927,29 @@ auth_failure(AUTH_MACH *m)
 {
 	grad_avp_t *pair;
 	char *cmd;
-	struct obstack stk;
 	
 	pair = grad_avl_find(m->user_reply, DA_AUTH_FAILURE_TRIGGER);
 	if (!pair) 
 		return;
 
-	obstack_init(&stk);
 	cmd = util_xlate(&m->msg_stack, pair->avp_strvalue, m->req->request);
 	switch (cmd[0]) {
 	case '(':
-		scheme_eval_boolean_expr(cmd);
+		scheme_eval_unspecified_expr(cmd);
 		break;
 
 	case '/':
 		radius_exec_command(cmd);
 		break;
 
+	default:
+		grad_log_req(L_ERR,
+			     m->req->request,
+			     _("Invalid Auth-Failure-Trigger value: %s"),
+			     cmd);
+		grad_log(L_INFO,
+			 _("The value of Auth-Failure-Trigger attribute must begin with '/' or '('."));
 	}
-	obstack_free(&stk, NULL);
 }
 
 void
